from .handle_google_searchai import *


from .handle_search_ai import HandleSearchAI

class FeatureHandlerMain:
    def __init__(self, chatbot, agent_type, query, web_search,system_prompt=""):
        self.chatbot = chatbot
        self.agent_type = agent_type
        self.query = query
        self.web_search = web_search
        self.system_prompt = system_prompt
        self.searchhandler = HandleSearchAI()

    def generate(self):
        match self.agent_type:
            case "QuestionAnswer":
                return self.chatbot.generator(self.query,
                                              system_prompt = self.system_prompt)
            case "FastSearchAI":
                return self.searchhandler.handle_search_ai(chatbot=self.chatbot, query=self.query, k=2)
            case "DeepSearchAI":
                return self.searchhandler.handle_search_ai(chatbot=self.chatbot, query=self.query, k=5)
            case "IntelligentSearchAI":
                return self.searchhandler.handle_search_ai(chatbot=self.chatbot, query=self.query, k=10)
            case _:
                return "None"



import streamlit as st

from ailitellm import yieldai
from .prompts import Prompts


class OmniCore:
    def __init__(self,model = 0,system_prompt = ""):
        self.system_prompt = system_prompt
        self.DEFAULT_MODELS = [
    "Qwen/Qwen2.5-72B-Instruct",
    "Qwen/QwQ-32B-Preview",
    "Qwen/Qwen2.5-Coder-32B-Instruct",
    "NousResearch/Hermes-3-Llama-3.1-8B",
    "microsoft/Phi-3.5-mini-instruct"
]
        self.current_model = model


    def generator(self,query,web_search= False,system_prompt = ""):
        messages = self._add_system_prompt(query,
                                        system_prompt=system_prompt)
        for resp in yieldai(messages_or_prompt=messages):
            if resp:
                yield resp

    def print_stream(self,query,web_search = False):
        for x in self.generator(query,web_search):
            print(x,end= "",flush=True)

    def _add_system_prompt(self, query,system_prompt= ""):
        user_query = Prompts.QUERY_PROMPT.format(query=query)
        return [
            {"role":"system","content":system_prompt or self.system_prompt},
            {"role":"user","content":user_query}
        ]

    def invoke(self,query,web_search=False):
        return self.generator(query)


from ..base import OmniCore
from visionlite import vision

class HandleSearchAI:
    def __init__(self,system_prompt=""):
        self.system_prompt = system_prompt

    def handle_search_ai(self,chatbot:OmniCore, query:str,k=1):
        context = vision(query,k=k,max_urls=5) # google search using chromedriver
        prompt = """
        <context>
        ### Search results:
        {context}
        </context>
        if context is empty , theh start by saying '# SEARCH RESULTS ARE EMPTY'
        user query: 
        
        {query}
        """.format(context=context, query=query)
        return chatbot.generator(prompt,system_prompt=self.system_prompt)




from .base import st


class OmniAiChatCSS:
    @staticmethod
    def render_main():
        st.markdown("""
            <style>
                /* Core variables */
                :root {
                    --background-primary: #FDFBF9;
                    --text-primary: #18181B;
                    --text-secondary: #71717A;
                    --border-color: #E4E4E7;
                }

                /* Base styles */
                .stApp {
                    background-color: var(--background-primary);
                    font-family: -apple-system, BlinkMacSystemFont, sans-serif;
                }

                /* Container adjustments */
                [data-testid="stAppViewContainer"] {
                    padding-top: 0 !important;
                }

                .block-container {
                    padding-top: 0 !important;
                    padding-bottom: 0 !important;
                    max-width: none;
                }

                /* Streamlit elements adjustments */
                .stSelectbox [data-testid="stMarkdown"] {
                    display: none;
                }

                /* Remove box styling from select boxes */
                .stSelectbox > div > div {
                    background: transparent !important;
                    border: none !important;
                    padding: 0 !important;
                }

                .stSelectbox {
                    color: var(--text-secondary);
                }

                /* Chat input styling */
                .stTextInput > div > div > input {
                    border: 1px solid var(--border-color);
                    border-radius: 8px;
                    padding: 0.75rem;
                    font-size: 0.95rem;
                }

                /* Message styling */
                [data-testid="stChatMessage"] {
                    background: transparent;
                    border: none;
                    padding: 1rem 0;
                    margin: 0;
                }

                /* Sidebar refinements */
                .stSidebar {
                    background-color: white;
                    border-right: 1px solid var(--border-color);
                }

                .stSidebar .block-container {
                    padding: 1.5rem 1rem;
                }

                /* Header/Navigation bar */
                .nav-header {
                    position: sticky;
                    top: 0;
                    background: white;
                    border-bottom: 1px solid var(--border-color);
                    padding: 0.5rem 1rem;
                    z-index: 100;
                    font-size: 0.9rem;
                    color: var(--text-secondary);
                }

                /* Remove unnecessary padding */
                .css-18e3th9, .css-1d391kg {
                    padding: 1rem 0;
                }

                /* File uploader styling */
                [data-testid="stFileUploader"] {
                    padding: 1rem 0;
                }

                /* Metrics button */
                .stButton > button[disabled] {
                    background: transparent !important;
                    border: none !important;
                    color: var(--text-secondary) !important;
                    padding: 0 !important;
                    font-size: 0.85rem;
                }

                /* Custom scrollbar */
                ::-webkit-scrollbar {
                    width: 6px;
                    height: 6px;
                }

                ::-webkit-scrollbar-track {
                    background: transparent;
                }

                ::-webkit-scrollbar-thumb {
                    background: #D4D4D8;
                    border-radius: 3px;
                }
            </style>
        """, unsafe_allow_html=True)

    @staticmethod
    def render_title():
        st.markdown("""
            <style>
                /* Minimal header */
                .stApp > header {
                    display: none;  /* Hide default header */
                }

                .app-header {
                    font-size: 1rem;
                    color: var(--text-primary);
                    padding: 0.5rem 1rem;
                    border-bottom: 1px solid var(--border-color);
                    background: white;
                    font-weight: 500;
                }
            </style>
            <div class="app-header">OmniAI</div>
            """, unsafe_allow_html=True)

    @staticmethod
    def render_chat_history_area():
        st.markdown("""
            <style>
                .chat-history {
                    height: calc(100vh - 180px);
                    overflow-y: auto;
                    padding: 1rem;
                    margin-bottom: 1rem;
                }

                .chat-message {
                    margin: 0.5rem 0;
                    padding: 0.5rem;
                    max-width: 90%;
                }

                .user-message {
                    margin-left: auto;
                    color: var(--text-primary);
                }

                .assistant-message {
                    margin-right: auto;
                    color: var(--text-primary);
                }
            </style>
        """, unsafe_allow_html=True)


import time

import aipromptlite

from .base import st
from datetime import datetime

from .baseprompt import BasePrompt
from .const import *
from .omni_mixin import OmniMixin
from .prompts import Prompts
from .streamlit_css import OmniAiChatCSS
from ..utils.prompt_names_fetcher import get_available_prompts

st.set_page_config(layout="wide", initial_sidebar_state='collapsed')

class OmniAIChatApp(OmniMixin):
    AVAILABLE_MODELS: List[str] = AVAILABLE_MODELS
    AGENT_TYPES: List[str] = AGENT_TYPES

    def __init__(self):
        self.sidebar = st.sidebar
        self.main_area = st.container()
        self.initialize_session_state()
        self.chunks_per_second = 0
        self.elapsed_time = 0

    def initialize_session_state(self):
        if "chats" not in st.session_state:
            st.session_state.chats = {}
        if "current_chat_id" not in st.session_state:
            st.session_state.current_chat_id = None
        if "chatbot" not in st.session_state:
            st.session_state.chatbot = None
        if "selected_model" not in st.session_state:
            st.session_state.selected_model = None
        if "sidebar_state" not in st.session_state:
            st.session_state.sidebar_state = "expanded"
        if "uploaded_file" not in st.session_state:
            st.session_state.uploaded_file = None
        if "agent_type" not in st.session_state:
            st.session_state.agent_type = self.AGENT_TYPES[0]
        if "web_search" not in st.session_state:
            st.session_state.web_search = False
        if "current_prompt" not in st.session_state:
            st.session_state.current_prompt = getattr(aipromptlite, "AILITE_X_CLAUDE_PROMPT")


    def render_sidebar(self):
        st.sidebar.title("Chat History")
        if st.sidebar.button("New Chat"):
            self.create_new_chat()

        for chat_id, chat_info in st.session_state.chats.items():
            if st.sidebar.button(f"{chat_info['name']} - {chat_info['timestamp'][:10]}"):
                st.session_state.current_chat_id = chat_id

        st.sidebar.write("Model Information")
        for k,v in MODELS_TITLE_MAP.items():
            st.sidebar.write(f'{k} - {v}')

    def create_new_chat(self):
        chat_id = datetime.now().strftime("%Y%m%d%H%M%S")
        st.session_state.chats[chat_id] = {
            "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M"),
            "name": f"Chat {len(st.session_state.chats) + 1}",
            "messages": []
        }
        st.session_state.current_chat_id = chat_id

    def render_chat_interface(self):
        with self.main_area:
            OmniAiChatCSS.render_main()
            OmniAiChatCSS.render_title()

            col1, col2 = st.columns([53,47],gap='small')

            self.chat_col = col1
            self.artifact_col = col2.container(height=ARTIFACT_COLUMN_HEIGHT,border=True)

            with self.chat_col:
                self.handle_user_input()
                # self.display_chat_messages()

    def display_chat_messages(self):
        if st.session_state.current_chat_id:
            for message in reversed(st.session_state.chats[st.session_state.current_chat_id]["messages"]):
                with st.chat_message(message["role"]):
                    if message["role"] == "user" and "file" in message:
                        st.write(f"Attached file: {message['file']}")
                    st.write(message["content"])

    def update_metrics(self):
        self.metrics_container.button(
            label=f"{self.chunks_per_second}/s, {self.elapsed_time}s",
            key="metrics_button_{}".format(datetime.now().timestamp()),
            disabled=True
        )

    def handle_user_input(self):
        # Create a container for the input area
        input_container = st.container()
        selection_container = st.container()

        with input_container:
            OmniAiChatCSS.render_chat_history_area()
            self.chat_history_area = st.container()
            self.chat_holder = self.chat_history_area.empty()

            col1, col2 = st.columns([6, 1], gap='small',vertical_alignment='bottom')

            with col1:
                if query := st.chat_input(placeholder="How can Claude help you today?"):
                    query+=""

            with col2:
                self.metrics_container = st.empty()
                self.update_metrics()



        with selection_container:
            sc1, sc2, sc3 = st.columns([1, 1, 1],gap='small')
            with sc1:
                models = list(MODELS_TITLE_MAP.keys()) + AVAILABLE_MODELS
                selected_model = st.selectbox("Select a model", models, label_visibility="hidden",
                                              key="model1")
                selected_model = MODELS_TITLE_MAP.get(selected_model, selected_model)


            with sc2:
                agent_type = st.selectbox("Agent type",self.AGENT_TYPES,
                                              label_visibility="hidden",
                                              key="agent_type")
                if st.session_state.agent_type is None or st.session_state.agent_type != agent_type:
                    st.session_state.agent_type = agent_type


            with sc3:
                prompt_name = st.selectbox("Select a model",
                                           get_available_prompts(),
                                           label_visibility="hidden",
                                           key="model3"
                                           )
                if prompt_name != 'DEFAULT_PROMPT':
                    st.session_state.current_prompt = getattr(aipromptlite, prompt_name)

            # with sc4:
            #     selected_model = st.selectbox("Select a model", self.AVAILABLE_MODELS, label_visibility="hidden",
            #                                   key="model4")



            if st.session_state.chatbot is None or st.session_state.selected_model != selected_model:
                system_prompt = self.get_system_prompt(st.session_state.agent_type)
                chatbot_instance = self.create_chat_instance(selected_model,system_prompt)
                # chatbot_instance.chatbot.new_conversation(
                #     modelIndex=self.AVAILABLE_MODELS.index(selected_model),
                #     system_prompt=system_prompt,
                #     switch_to=True)
                st.session_state.chatbot = chatbot_instance
                st.session_state.selected_model = selected_model

            uploaded_file = st.file_uploader("file uploading", type=["pdf"], key="file_uploader",
                                             label_visibility="collapsed")

            if uploaded_file is not None:
                st.session_state.uploaded_file = uploaded_file
                st.success(f"File {uploaded_file.name} uploaded successfully!")


        if query or getattr(st.session_state, 'uploaded_file', None):
            if not st.session_state.current_chat_id:
                self.create_new_chat()

            current_chat = st.session_state.chats[st.session_state.current_chat_id]

            if st.session_state.uploaded_file:
                file_content = st.session_state.uploaded_file.read()
                file_name = st.session_state.uploaded_file.name
                file_extension = file_name.lower().split('.')[-1]

                if file_extension in ['pdf', 'png', 'jpg', 'jpeg', 'gif', 'bmp', 'tiff']:
                    with st.spinner("Analyzing pdf ..."):
                        query = self.handle_files(query,file_content,file_extension)

                current_chat["messages"].append({
                    "role": "user",
                    "content": f"Uploaded file: {file_name}",
                    "file": file_name
                })
                st.session_state.uploaded_file = None

            if query:
                current_chat["messages"].append({"role": "user", "content": query})

            self.process_ai_response(query,web_search = st.session_state.web_search,
                                     system_prompt=st.session_state.current_prompt,
                                     agent_type = st.session_state.agent_type)

    def process_ai_response(self, query: str, web_search=False,
                            agent_type =None,
                            system_prompt=""):
        system_prompt = self.add_time_and_artifact_to_system_prompt(system_prompt=system_prompt)

        chat_placeholder = st.empty()
        artifact_placeholder = self.artifact_col.empty()

        response_generator = self.get_chat_response(
            chatbot=st.session_state.chatbot,
            agent_type=agent_type,
            query=query,
            web_search=web_search,
            system_prompt=system_prompt
        )
        # Initialize chunk counting and timing
        start_time = time.time()
        chunk_count = 0

        def chunk_counting_stream(generator):
            nonlocal chunk_count
            for chunk, flag in self.data_stream(generator):
                chunk_count += 1
                yield chunk, flag

        chat_content, artifact_content = self.update_chat_col(
            chunk_counting_stream(response_generator),
            chat_placeholder,
            artifact_placeholder,
            self.chat_holder
        )

        # Calculate chunks per second
        end_time = time.time()
        elapsed_time = end_time - start_time
        chunks_per_second = chunk_count / elapsed_time if elapsed_time > 0 else 0

        # Display chunks per second
        self.chunks_per_second = int(chunks_per_second)
        self.elapsed_time = int(elapsed_time)
        self.total_tokens = chunk_count
        st.sidebar.text(f"Tokens/s : {self.chunks_per_second}")
        st.sidebar.text(f"Inference: {int(elapsed_time)}s")
        st.sidebar.text(f"Total tokens: {chunk_count}")
        self.update_metrics()

        current_chat = st.session_state.chats[st.session_state.current_chat_id]
        current_chat["messages"].append({"role": "assistant", "content": chat_content})
        if artifact_content:
            current_chat["messages"].append({"role": "artifact", "content": artifact_content})

    def run(self):
        self.render_sidebar()
        self.render_chat_interface()

    def get_system_prompt(self, agent_type):
        if agent_type == "AIResearcher":
            return Prompts.REASONING_SBS_PROMPT
        else:
            return Prompts.WORKING_SYSTEM_PROMPT1

    def add_time_and_artifact_to_system_prompt(self, system_prompt):
        return BasePrompt.TODAY_DATE + system_prompt + "\n" + BasePrompt.ARTIFACT


import subprocess
import time

def serve():
    print("🚀 Starting OmniAI Chat App...")
    print("🔧 Initializing...")

    # Start the Streamlit server using subprocess
    process = subprocess.Popen(
        ["streamlit", "run", "core/streamlit_app.py", "--server.headless=true"],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd="aichatlite"  # Ensure the current working directory is correct
    )

    print("🌐 Starting Streamlit server...")

    # Capture and print the output from the subprocess
    try:
        while True:
            output = process.stdout.readline()
            if output == b'' and process.poll() is None:
                break
            if output:
                print(output.decode().strip())
    except KeyboardInterrupt:
        print("Shutting down the server...")
        process.terminate()
        process.wait()
        print("Server has been shut down.")

    # Capture and print any errors
    error_output = process.stderr.read()
    if error_output:
        print("Error output from Streamlit server:")
        print(error_output.decode().strip())

    # Ensure the process continues to run
    process.communicate()


from datetime import datetime


class BasePrompt:
    TODAY_DATE = f"""The current date is {datetime.now().strftime("%Y-%m-%d")}.\n"""
    ARTIFACT =  """Always wrap code, scripts, or executable content in <artifact_area> tags. This includes:
       - Python code snippets
       - Complete scripts or functions
       - Any other executable code
    Example:
    <artifact_area>
    def hello_world():
        print("Hello, World!")
    </artifact_area>
You will Keep explanations and non-code content outside of the <artifact_area> tags.
DO NOT FORGET TO WRAP PYTHON CODE TO WRAP IN <artifact_area> tags."""


import aipromptlite

def get_available_prompts():
    prompt_names = [_ for _ in dir(aipromptlite) if _.isupper()]
    prompt_names.sort()
    return ['DEFAULT_PROMPT']+ prompt_names
# def get_available_prompts():
#     prompt_groups = {
#         'Code': ['PYTHON_CODER_PROMPT', 'CLAUDE_CODER_PROMPT'],
#         'Reason': ['AI_TECH_LEAD_PROMPT', 'BRAINSTORM_BUDDY_PROMPT'],
#         'Explain': ['PREDICTIVE_QUESTION_PROMPT', 'AILITE_CLAUDE_SYSTEM_PROMPT']
#     }
#     return prompt_groups
